part of masamune_purchase_stripe;

final stripeConnectPaymentModelProvider = ChangeNotifierProvider(
  (_) => StripeConnectPaymentModel(),
);

final stripeConnectPaymentCollectionProvider =
    ChangeNotifierProvider.family<FirestoreDynamicCollectionModel, String>(
  (_, uid) => FirestoreDynamicCollectionModel(
      "${StripeConnectCore._userPath}/$uid/${StripeConnectCore._paymentPath}"),
);

class StripeConnectPaymentModel
    extends ValueModel<FirestoreDynamicCollectionModel?>
    with IterableMixin<FirestoreDynamicDocumentModel> {
  StripeConnectPaymentModel() : super(null);

  @override
  Iterator<FirestoreDynamicDocumentModel> get iterator =>
      value?.iterator ?? <FirestoreDynamicDocumentModel>[].iterator;

  @override
  @protected
  void initState() {
    super.initState();
    _initialize();
  }

  Future<void> reload() async {
    if (value == null) {
      return;
    }
    await value!.load();
  }

  bool get exists {
    if (value == null) {
      return false;
    }
    return value.isNotEmpty;
  }

  /// Returns itself after the load finishes.
  Future<void>? get loading => _loadingCompleter?.future;
  Completer<void>? _loadingCompleter;

  /// Returns itself after the save finishes.
  Future<void>? get saving =>
      _savingCompleter?.future ?? _createCompleter?.future;
  Completer<String>? _createCompleter;
  Completer<void>? _savingCompleter;

  /// If this value is `true`,
  /// the change will be notified when [value] itself is changed.
  @override
  bool get notifyOnChangeValue => false;

  Future<void> _initialize() async {
    if (value != null) {
      return;
    }
    if (!StripeConnectCore.isInitialized) {
      await StripeConnectCore._future;
      if (!StripeConnectCore.isInitialized) {
        throw Exception("StripeConnectCore has not been initialized.");
      }
    }
    value = StripeConnectCore._paymentModel;
    value?.addListener(_handledOnUpdate);
    notifyListeners();
  }

  void _handledOnUpdate() {
    notifyListeners();
  }

  Future<String> create(
    BuildContext context, {
    required void Function(
            Widget webView, VoidCallback onSuccess, VoidCallback onCancel)
        builder,
    bool popWhenClosedByWebview = false,
  }) async {
    if (_createCompleter != null) {
      return _createCompleter!.future;
    }
    _createCompleter = Completer<String>();
    try {
      await _initialize();
      final uid = StripeConnectCore._debugUserId ?? userId;
      if (uid.isEmpty) {
        throw Exception("You are not logged in. Please log in once.");
      }
      final functions =
          read(functionsDocumentProvider(StripeConnectCore._serverPath));
      final res = await functions.call(
        parameters: {
          "mode": _StripeConnectAPIMode.createPayment.id,
          "userId": uid,
          "successUrl":
              "${StripeConnectCore._callbackUrl}/create_payment/success",
          "cancelUrl":
              "${StripeConnectCore._callbackUrl}/create_payment/cancel",
        },
      );
      if (res.isEmpty) {
        throw Exception("Response is invalid.");
      }
      final endpoint = res.get("endpoint", "");
      final customerId = res.get("customerId", "");
      if (endpoint.isEmpty || customerId.isEmpty) {
        throw Exception("Response is invalid.");
      }
      final onSuccess = () {
        _createCompleter?.complete(customerId);
        _createCompleter = null;
      };
      final onCancel = () {
        _createCompleter?.completeError(StripeConnectCancelException());
        _createCompleter = null;
      };
      final webView = _StripeConnectWebview(Uri.parse(endpoint),
          shouldOverrideUrlLoading: (controller, url) {
        final path =
            url.trimQuery().replaceAll(StripeConnectCore._callbackUrl, "");
        switch (path) {
          case "/create_payment/success":
            if (popWhenClosedByWebview) {
              context.navigator.pop();
            }
            onSuccess.call();
            return NavigationActionPolicy.CANCEL;
          case "/create_payment/cancel":
            if (popWhenClosedByWebview) {
              context.navigator.pop();
            }
            onCancel.call();
            return NavigationActionPolicy.CANCEL;
        }
        return NavigationActionPolicy.ALLOW;
      }, onCloseWindow: (controller) {
        onCancel.call();
      });
      builder.call(webView, onSuccess, onCancel);
      await _createCompleter!.future;
      notifyListeners();
      return customerId;
    } catch (e) {
      _createCompleter?.completeError(e);
      _createCompleter = null;
      rethrow;
    } finally {
      _createCompleter?.complete("");
      _createCompleter = null;
    }
  }

  Future<void> setDefault(String paymentId) async {
    if (_savingCompleter != null) {
      return _savingCompleter!.future;
    }
    _savingCompleter = Completer<void>();
    try {
      await _initialize();
      final uid = StripeConnectCore._debugUserId ?? userId;
      if (uid.isEmpty) {
        throw Exception("You are not logged in. Please log in once.");
      }
      final functions =
          read(functionsDocumentProvider(StripeConnectCore._serverPath));
      final res = await functions.call(
        parameters: {
          "mode": _StripeConnectAPIMode.setDefaultPayment.id,
          "userId": uid,
          "paymentId": paymentId,
        },
      );
      if (res.isEmpty) {
        throw Exception("Response is invalid.");
      }
      final success = res.get("success", false);
      if (!success) {
        throw Exception("Response is invalid.");
      }
      notifyListeners();
      _savingCompleter?.complete();
      _savingCompleter = null;
    } catch (e) {
      _savingCompleter?.completeError(e);
      _savingCompleter = null;
      rethrow;
    } finally {
      _savingCompleter?.complete();
      _savingCompleter = null;
    }
  }

  Future<void> delete(String paymentId) async {
    if (_savingCompleter != null) {
      return _savingCompleter!.future;
    }
    _savingCompleter = Completer<void>();
    try {
      await _initialize();
      final uid = StripeConnectCore._debugUserId ?? userId;
      if (uid.isEmpty) {
        throw Exception("You are not logged in. Please log in once.");
      }
      final functions =
          read(functionsDocumentProvider(StripeConnectCore._serverPath));
      final res = await functions.call(
        parameters: {
          "mode": _StripeConnectAPIMode.deletePayment.id,
          "userId": uid,
          "paymentId": paymentId,
        },
      );
      if (res.isEmpty) {
        throw Exception("Response is invalid.");
      }
      final success = res.get("success", false);
      if (!success) {
        throw Exception("Response is invalid.");
      }
      notifyListeners();
      _savingCompleter?.complete();
      _savingCompleter = null;
    } catch (e) {
      _savingCompleter?.completeError(e);
      _savingCompleter = null;
      rethrow;
    } finally {
      _savingCompleter?.complete();
      _savingCompleter = null;
    }
  }
}
